@@ -76,13 +76,18 @@ enum srv_state {
 enum srv_admin {
 	SRV_ADMF_FMAINT    = 0x01,        /* the server was explicitly forced into maintenance */
 	SRV_ADMF_IMAINT    = 0x02,        /* the server has inherited the maintenance status from a tracked server */
-	SRV_ADMF_MAINT     = 0x23,        /* mask to check if any maintenance flag is present */
 	SRV_ADMF_CMAINT    = 0x04,        /* the server is in maintenance because of the configuration */
 	SRV_ADMF_FDRAIN    = 0x08,        /* the server was explicitly forced into drain state */
 	SRV_ADMF_IDRAIN    = 0x10,        /* the server has inherited the drain status from a tracked server */
 	SRV_ADMF_DRAIN     = 0x18,        /* mask to check if any drain flag is present */
 	SRV_ADMF_RMAINT    = 0x20,        /* the server is down because of an IP address resolution failure */
-	SRV_ADMF_HMAINT    = 0x40,        /* the server FQDN has been set from socket stats */
+
+	SRV_ADMF_MAINT     = 0x23,        /* mask to check if any maintenance flag except CMAINT is present */
+
+	SRV_ADMF_FQDN_CHANGED = 0x40,     /* Special value: set (and never removed) if the server fqdn has
+	                                   * changed (from cli or resolvers) since its initial value from
+	                                   * config. This flag is exported and restored through state-file
+					   */
 } __attribute__((packed));
 
 /* options for servers' "init-addr" parameter
@@ -134,7 +139,7 @@ enum srv_initaddr {
 #define SRV_STATE_FILE_MAX_FIELDS 25
 #define SRV_STATE_FILE_MIN_FIELDS_VERSION_1 20
 #define SRV_STATE_FILE_MAX_FIELDS_VERSION_1 25
-#define SRV_STATE_LINE_MAXLEN 512
+#define SRV_STATE_LINE_MAXLEN 2000
 
 /* server flags -- 32 bits */
 #define SRV_F_BACKUP       0x0001        /* this server is a backup server */
@@ -286,6 +291,7 @@ struct server {
 	unsigned int next_takeover;             /* thread ID to try to steal connections from next time */
 
 	struct eb_root pendconns;		/* pending connections */
+	struct mt_list sess_conns;		/* list of private conns managed by a session on this server */
 
 	/* Element below are usd by LB algorithms and must be doable in
 	 * parallel to other threads reusing connections above.
