@@ -734,6 +734,21 @@ static int cli_parse_request(struct appctx *appctx)
 	for (; i < MAX_CLI_ARGS + 1; i++)
 		args[i] = p;
 
+	if (appctx->st1 & APPCTX_CLI_ST1_SHUT_EXPECTED) {
+		/* The previous command line was finished by a \n in non-interactive mode.
+		 * It should not be followed by another command line. In non-interactive mode,
+		 * only one line should be processed. Because of a bug, it is not respected.
+		 * So emit a warning, only once in the process life, to warn users their script
+		 * must be updated.
+		 */
+		appctx->st1 &= ~APPCTX_CLI_ST1_SHUT_EXPECTED;
+		if (ONLY_ONCE()) {
+			ha_warning("Commands sent to the CLI were chained using a new line character while in non-interactive mode."
+				   " This is not reliable, not officially supported and will not be supported anymore in future versions. "
+				   "Please use ';' to delimit commands instead.");
+		}
+	}
+
 	kw = cli_find_kw(args);
 	if (!kw ||
 	    (kw->level & ~appctx->cli_level & ACCESS_MASTER_ONLY) ||
@@ -781,13 +796,14 @@ fail:
 static int cli_output_msg(struct channel *chn, const char *msg, int severity, int severity_output)
 {
 	struct buffer *tmp;
-
-	if (likely(severity_output == CLI_SEVERITY_NONE))
-		return ci_putblk(chn, msg, strlen(msg));
+	struct ist imsg;
 
 	tmp = get_trash_chunk();
 	chunk_reset(tmp);
 
+	if (likely(severity_output == CLI_SEVERITY_NONE))
+		goto send_it;
+
 	if (severity < 0 || severity > 7) {
 		ha_warning("socket command feedback with invalid severity %d", severity);
 		chunk_printf(tmp, "[%d]: ", severity);
@@ -804,9 +820,19 @@ static int cli_output_msg(struct channel *chn, const char *msg, int severity, in
 				ha_warning("Unrecognized severity output %d", severity_output);
 		}
 	}
-	chunk_appendf(tmp, "%s", msg);
+ send_it:
+	/* the vast majority of messages have their trailing LF but a few are
+	 * still missing it, and very rare ones might even have two. For this
+	 * reason, we'll first delete the trailing LFs if present, then
+	 * systematically append one.
+	 */
+	for (imsg = ist(msg); imsg.len > 0 && imsg.ptr[imsg.len - 1] == '\n'; imsg.len--)
+		;
 
-	return ci_putblk(chn, tmp->area, strlen(tmp->area));
+	chunk_istcat(tmp, imsg);
+	chunk_istcat(tmp, ist("\n"));
+
+	return ci_putchk(chn, tmp);
 }
 
 /* This I/O handler runs as an applet embedded in a stream interface. It is
@@ -825,6 +851,7 @@ static void cli_io_handler(struct appctx *appctx)
 	struct bind_conf *bind_conf = strm_li(si_strm(si))->bind_conf;
 	int reql;
 	int len;
+	int lf = 0;
 
 	if (unlikely(si->state == SI_ST_DIS || si->state == SI_ST_CLO))
 		goto out;
@@ -898,29 +925,15 @@ static void cli_io_handler(struct appctx *appctx)
 				continue;
 			}
 
-			if (!(appctx->st1 & APPCTX_CLI_ST1_PAYLOAD)) {
-				/* seek for a possible unescaped semi-colon. If we find
-				 * one, we replace it with an LF and skip only this part.
-				 */
-				for (len = 0; len < reql; len++) {
-					if (str[len] == '\\') {
-						len++;
-						continue;
-					}
-					if (str[len] == ';') {
-						str[len] = '\n';
-						reql = len + 1;
-						break;
-					}
-				}
-			}
+			if (str[reql-1] == '\n')
+				lf = 1;
 
 			/* now it is time to check that we have a full line,
 			 * remove the trailing \n and possibly \r, then cut the
 			 * line.
 			 */
 			len = reql - 1;
-			if (str[len] != '\n') {
+			if (str[len] != '\n' && str[len] != ';') {
 				appctx->st0 = CLI_ST_END;
 				continue;
 			}
@@ -950,8 +963,9 @@ static void cli_io_handler(struct appctx *appctx)
 					/* NB: cli_sock_parse_request() may have put
 					 * another CLI_ST_O_* into appctx->st0.
 					 */
-
 					appctx->st1 &= ~APPCTX_CLI_ST1_PAYLOAD;
+					if (!(appctx->st1 & APPCTX_CLI_ST1_PROMPT) && lf)
+						appctx->st1 |= APPCTX_CLI_ST1_SHUT_EXPECTED;
 				}
 			}
 			else {
@@ -968,6 +982,8 @@ static void cli_io_handler(struct appctx *appctx)
 					/* no payload, the command is complete: parse the request */
 					cli_parse_request(appctx);
 					chunk_reset(appctx->chunk);
+					if (!(appctx->st1 & APPCTX_CLI_ST1_PROMPT) && lf)
+						appctx->st1 |= APPCTX_CLI_ST1_SHUT_EXPECTED;
 				}
 			}
 
@@ -2632,13 +2648,10 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 
 		s->target = NULL;
 
-		/* only release our endpoint if we don't intend to reuse the
-		 * connection.
-		 */
-		if (!si_conn_ready(&s->si[1])) {
-			si_release_endpoint(&s->si[1]);
-			s->srv_conn = NULL;
-		}
+
+		/* Always release our endpoint */
+		si_release_endpoint(&s->si[1]);
+		s->srv_conn = NULL;
 
 		sockaddr_free(&s->target_addr);
 
@@ -2664,7 +2677,7 @@ int pcli_wait_for_response(struct stream *s, struct channel *rep, int an_bit)
 		s->target = NULL;
 		/* re-init store persistence */
 		s->store_count = 0;
-		s->uniq_id = global.req_count++;
+		s->uniq_id = _HA_ATOMIC_FETCH_ADD(&global.req_count, 1);
 
 		s->req.flags |= CF_READ_DONTWAIT; /* one read is usually enough */
 
