@@ -649,11 +649,18 @@ struct pattern *pat_match_beg(struct sample *smp, struct pattern_expr *expr, int
 
 		if (smp->data.u.str.data < smp->data.u.str.size) {
 			/* we may have to force a trailing zero on the test pattern and
-			 * the buffer is large enough to accommodate it.
+			 * the buffer is large enough to accommodate it. If the flag
+			 * CONST is set, duplicate the string
 			 */
 			prev = smp->data.u.str.area[smp->data.u.str.data];
-			if (prev)
-				smp->data.u.str.area[smp->data.u.str.data] = '\0';
+			if (prev) {
+				if (smp->flags & SMP_F_CONST) {
+					if (!smp_dup(smp))
+						return NULL;
+				} else {
+					smp->data.u.str.area[smp->data.u.str.data] = '\0';
+				}
+			}
 		}
 		else {
 			/* Otherwise, the sample is duplicated. A trailing zero
@@ -1753,28 +1760,18 @@ int pat_ref_set(struct pat_ref *ref, const char *key, const char *value, char **
 {
 	struct pat_ref_elt *elt;
 	int found = 0;
-	char *_merr;
-	char **merr;
-
-	if (err) {
-		merr = &_merr;
-		*merr = NULL;
-	}
-	else
-		merr = NULL;
 
 	/* Look for pattern in the reference. */
 	list_for_each_entry(elt, &ref->head, list) {
 		if (strcmp(key, elt->pattern) == 0) {
-			if (!pat_ref_set_elt(ref, elt, value, merr)) {
-				if (err && merr) {
-					if (!found) {
-						*err = *merr;
-					} else {
-						memprintf(err, "%s, %s", *err, *merr);
-						ha_free(merr);
-					}
-				}
+			char *tmp_err = NULL;
+
+			if (!pat_ref_set_elt(ref, elt, value, &tmp_err)) {
+				if (err)
+					*err = tmp_err;
+				else
+					ha_free(&tmp_err);
+				return 0;
 			}
 			found = 1;
 		}
@@ -2072,12 +2069,6 @@ int pat_ref_purge_range(struct pat_ref *ref, uint from, uint to, int budget)
 	list_for_each_entry(expr, &ref->pat, list)
 		HA_RWLOCK_WRUNLOCK(PATEXP_LOCK, &expr->lock);
 
-#if defined(HA_HAVE_MALLOC_TRIM)
-	if (done) {
-		malloc_trim(0);
-	}
-#endif
-
 	return done;
 }
 
@@ -2174,19 +2165,14 @@ struct pattern_expr *pattern_new_expr(struct pattern_head *head, struct pat_ref
 		expr->ref = ref;
 
 		HA_RWLOCK_INIT(&expr->lock);
-
-		/* We must free this pattern if it is no more used. */
-		list->do_free = 1;
 	}
 	else {
-		/* If the pattern used already exists, it is already linked
-		 * with ref and we must not free it.
-		 */
-		list->do_free = 0;
 		if (reuse)
 			*reuse = 1;
 	}
 
+	HA_ATOMIC_INC(&expr->refcount);
+
 	/* The new list element reference the pattern_expr. */
 	list->expr = expr;
 
@@ -2548,7 +2534,7 @@ void pattern_prune(struct pattern_head *head)
 
 	list_for_each_entry_safe(list, safe, &head->head, list) {
 		LIST_DELETE(&list->list);
-		if (list->do_free) {
+		if (HA_ATOMIC_SUB_FETCH(&list->expr->refcount, 1) == 0) {
 			LIST_DELETE(&list->expr->list);
 			HA_RWLOCK_WRLOCK(PATEXP_LOCK, &list->expr->lock);
 			head->prune(list->expr);
